package com.winhc.bigdata.flink.java.entity;
import com.google.common.collect.Maps;
import org.apache.hadoop.hbase.util.Bytes;
import java.util.Collection;
import java.util.Map;

/**
 * @author ZhangJi
 * @since 2021-10-11 15:33
 */
public class PutCollection {


    public Map<String, Map<String, SealedPut>> getData() {
        return data;
    }

    public void setData(Map<String, Map<String, SealedPut>> data) {
        this.data = data;
    }

    private Map<String, Map<String, SealedPut>> data = Maps.newHashMap();

    public void putAll(String tableName, Collection<SealedPut> puts) {
        data.putIfAbsent(tableName,Maps.newHashMap());
        if (!data.containsKey(tableName)) {
            data.put(tableName, Maps.newHashMap());
        }
        final Map<String, SealedPut> table = data.get(tableName);
        for (SealedPut put : puts) {
            final String hexRow = Bytes.toHex(put.getRowKey());
            if (!table.containsKey(hexRow)) {
                table.put(hexRow, put);
            } else {
                table.put(
                        hexRow,
                        table.get(hexRow).appendSealedPut(put)
                );
            }
        }
    }

    public void put(String tableName, SealedPut put) {
        data.putIfAbsent(tableName,Maps.newHashMap());
//        if (!data.containsKey(tableName)) {
//            data.put(tableName, Maps.newHashMap());
//        }
        final Map<String, SealedPut> table = data.get(tableName);
        final String hexRow = Bytes.toHex(put.getRowKey());
        if (!table.containsKey(hexRow)) {
            table.put(hexRow, put);
        } else {
            table.put(
                    hexRow,
                    table.get(hexRow).appendSealedPut(put)
            );
        }
    }


    /**
     * 合并多个PutCollection
     *
     * @param cs
     * @return
     */
    public static PutCollection merge(PutCollection... cs) {
        PutCollection c0 = new PutCollection();
        for (PutCollection ci : cs) {
            for (Map.Entry<String, Map<String, SealedPut>> entry : ci.data.entrySet()) {
                c0.putAll(entry.getKey(), entry.getValue().values());
            }
        }
        return c0;
    }

    public static PutCollection create(String tableName, Collection<SealedPut> puts) {
        PutCollection result = new PutCollection();
        result.putAll(tableName, puts);
        return result;
    }

    public static PutCollection create(String tableName, SealedPut put) {
        PutCollection result = new PutCollection();
        result.put(tableName, put);
        return result;
    }
}

