package com.winhc.bigdata.flink.java.sink;

import com.alibaba.ververica.connectors.common.sink.TupleOutputFormatSinkFunction;
import com.alibaba.ververica.connectors.odps.sink.OdpsOutputFormat;
import com.aliyun.odps.Column;
import com.aliyun.odps.Odps;
import com.aliyun.odps.Table;
import com.aliyun.odps.account.AliyunAccount;
import com.aliyun.odps.type.TypeInfo;
import com.winhc.bigdata.flink.java.constant.EnvConst;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.table.api.DataTypes;
import org.apache.flink.table.api.TableSchema;
import org.apache.flink.table.descriptors.DescriptorProperties;
import org.apache.flink.table.types.DataType;
import org.apache.flink.types.Row;

/**
 * @author: XuJiakai
 * 2021/8/31 16:12
 */
public class OdpsSinkBuilder4J {

    /**
     * 如果是动态分区只需要指定 ds
     *
     * @param tableName
     * @param partition ds  或 ds=20210903
     * @return
     */
    public static SinkFunction<Tuple2<Boolean, Row>> buildSinkFunction(String tableName, String partition) {
        String endPoint = EnvConst.getValue("odps.url.end-point");
        String tunnelEndPoint = EnvConst.getValue("odps.url.tunnel-end-point");
        String accessId = EnvConst.getValue("odps.prod-account.access-id");
        String accessKey = EnvConst.getValue("odps.prod-account.access-key");

        String project = EnvConst.getValue("odps.default-project");
        String tn = tableName;
        if (tableName.contains(".")) {
            String[] strings = tableName.split("\\.");
            project = strings[0];
            tn = strings[1];
        }


        DescriptorProperties properties = new DescriptorProperties();
        properties.putString("endpoint", endPoint);
        properties.putString("tunnelEndpoint", tunnelEndPoint);
        properties.putString("project", project);
        properties.putString("tablename", tn);
        properties.putString("accessid", accessId);
        properties.putString("accesskey", accessKey);
        if (partition != null) {
            properties.putString("partition", partition);
        }

        AliyunAccount account = new AliyunAccount(accessId, accessKey);
        Odps odps = new Odps(account);
        odps.setEndpoint(endPoint);
        odps.setDefaultProject(project);
        Table table = odps.tables().get(tableName);


        TableSchema.Builder builder = TableSchema.builder();
        for (Column column : table.getSchema().getColumns()) {
            builder.field(column.getName(), switchOdpsType(column.getTypeInfo()));
        }
        if (StringUtils.isNotBlank(partition) && !partition.contains("=")) {
            builder.field("ds", DataTypes.STRING());
        }

        return new TupleOutputFormatSinkFunction<>(
                new OdpsOutputFormat(builder.build(), properties)
        );
    }

    private static DataType switchOdpsType(TypeInfo typeInfo) {
        switch (typeInfo.getOdpsType()) {
            case STRING:
                return DataTypes.STRING();
            case INT:
                return DataTypes.INT();
            case DOUBLE:
                return DataTypes.DOUBLE();
            case BIGINT:
                return DataTypes.BIGINT();
            default:
                throw new RuntimeException("data type is not fount !" + typeInfo.getTypeName());
        }
    }

}
