package com.winhc.bigdata.flink.java.source;

import com.alibaba.ververica.connectors.odps.ODPSStreamSource;
import com.alibaba.ververica.connectors.odps.OdpsConf;
import com.alibaba.ververica.connectors.odps.OdpsOptions;
import com.alibaba.ververica.connectors.odps.schema.ODPSColumn;
import com.aliyun.odps.Column;
import com.aliyun.odps.Odps;
import com.aliyun.odps.Table;
import com.aliyun.odps.account.AliyunAccount;
import com.aliyun.odps.utils.StringUtils;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.table.data.GenericRowData;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/8/30 17:29
 */
public class OdpsSourceBuilder4J {
    private Configuration conf;
    private Odps odps;
    private String tableName;
    private List<String> ds;
    private String accessId;
    private String accessKey;
    private String endPoint;
    private String projectName;
    private String tunnelEndpoint;

    public OdpsSourceBuilder4J(Configuration conf) {
        this.conf = conf;
        this.tableName = conf.getString(OdpsOptions.TABLE_NAME);
        this.ds = Arrays.stream(conf.getString(OdpsOptions.PARTITION, "").split(","))
                .filter(StringUtils::isNotBlank).collect(Collectors.toList());
        this.accessId = conf.getString(OdpsOptions.ACCESS_ID);
        this.accessKey = conf.getString(OdpsOptions.ACCESS_KEY);
        this.endPoint = conf.getString(OdpsOptions.END_POINT);
        this.projectName = conf.getString(OdpsOptions.PROJECT_NAME);
        this.tunnelEndpoint = conf.getString(OdpsOptions.TUNNEL_END_POINT);


        AliyunAccount account = new AliyunAccount(accessId, accessKey);
        Odps odps = new Odps(account);
        odps.setEndpoint(endPoint);
        odps.setDefaultProject(projectName);
        this.odps = odps;
    }

    public ODPSStreamSource buildSourceFunction() {
        OdpsConf odpsConf = new OdpsConf(accessId, accessKey, endPoint, projectName, tunnelEndpoint);
        Table table = odps.tables().get(tableName);

        List<Column> columns = table.getSchema().getColumns();
        ODPSColumn[] odpsColumns = columns.stream().map(c -> new ODPSColumn(c.getName(), c.getTypeInfo().getOdpsType())).collect(Collectors.toList())
                .toArray(new ODPSColumn[columns.size()]);

        TypeInformation info = TypeInformation.of(GenericRowData.class);
        ODPSStreamSource source = new ODPSStreamSource(odpsConf, tableName, odpsColumns, ds, info, 1000L, 1000L);
        return source;
    }
}
