package com.winhc.bigdata.flink.java.utils;

import org.apache.commons.lang3.StringUtils;
import org.yaml.snakeyaml.Yaml;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/8/31 09:32
 */
public class EnvironmentProperties {
    private static final String fileName = "env.yml";

    private Map<String, String> source;

    public String getValue(String key) {
        return source.getOrDefault(key, null);
    }

    public Map<String, String> getMapByPrefix(String prefix) {
        return source.entrySet().stream().filter(e -> e.getKey().startsWith(prefix)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (e1, e2) -> e1));
    }

    public EnvironmentProperties() {
        Map<String, String> source = getYmlByFileName(null, null);
        String profile = source.getOrDefault("profile.activate", null);
        if (StringUtils.isNotBlank(profile)) {
            source = getYmlByFileName("env-" + profile + ".yml", source);
        }
        this.source = source;
    }

    public Map<String, String> getSource() {
        return source;
    }


    /**
     * 根据文件名获取yml的文件内容
     *
     * @return
     */
    private static Map<String, String> getYmlByFileName(String file, Map<String, String> source) {
        if (source == null) {
            source = new HashMap<>();
        }
        if (file == null)
            file = fileName;
        InputStream in = EnvironmentProperties.class.getClassLoader().getResourceAsStream(file);
        Yaml props = new Yaml();
        Object obj = props.loadAs(in, Map.class);
        Map<String, Object> param = (Map<String, Object>) obj;

        for (Map.Entry<String, Object> entry : param.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            if (val instanceof Map) {
                forEachYaml(source, key, (Map<String, Object>) val);
            } else {
                source.put(key, val.toString());
            }
        }
        return source;
    }

    /**
     * 遍历yml文件，获取map集合
     *
     * @param key_str
     * @param obj
     * @return
     */
    private static Map<String, String> forEachYaml(Map<String, String> source, String key_str, Map<String, Object> obj) {
        for (Map.Entry<String, Object> entry : obj.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            String str_new = "";
            if (StringUtils.isNotBlank(key_str)) {
                str_new = key_str + "." + key;
            } else {
                str_new = key;
            }
            if (val instanceof Map) {
                forEachYaml(source, str_new, (Map<String, Object>) val);
            } else {
                source.put(str_new, val.toString());
            }
        }
        return source;
    }


    public static void main(String[] args) throws IOException {
        EnvironmentProperties environmentProperties = new EnvironmentProperties();

        String value = environmentProperties.getValue("es.a");
        System.out.println(value);
    }

}
