package com.winhc.bigdata.flink.config

import com.winhc.bigdata.flink.java.constant.EnvConst
import org.apache.flink.elasticsearch6.shaded.org.apache.http.HttpHost
import org.apache.flink.elasticsearch6.shaded.org.apache.http.auth.{AuthScope, UsernamePasswordCredentials}
import org.apache.flink.elasticsearch6.shaded.org.apache.http.client.config.RequestConfig
import org.apache.flink.elasticsearch6.shaded.org.apache.http.impl.client.BasicCredentialsProvider
import org.apache.flink.elasticsearch6.shaded.org.apache.http.impl.nio.client.HttpAsyncClientBuilder
import org.apache.flink.elasticsearch6.shaded.org.elasticsearch.client.{RestClient, RestClientBuilder, RestHighLevelClient}

import java.time.Duration

/**
 * @author: XuJiakai
 * @date: 2021/8/31 13:51
 */
object ElasticsearchConfig {
  def getConfig(clusterName: String): (Array[HttpHost], RestClientBuilder.HttpClientConfigCallback , RestClientBuilder.RequestConfigCallback) = {
    val hosts: Array[HttpHost] = EnvConst.getValue("winhc.elasticsearch.rest." + clusterName + ".uris")
      .split(",").map(h => HttpHost.create(h))
    val username = EnvConst.getValue("winhc.elasticsearch.rest." + clusterName + ".username")
    val password = EnvConst.getValue("winhc.elasticsearch.rest." + clusterName + ".password")
    val connectionTimeout = EnvConst.getValue("winhc.elasticsearch.rest." + clusterName + ".connection-timeout", "100s")
    val readTimeout = EnvConst.getValue("winhc.elasticsearch.rest." + clusterName + ".read-timeout", "300s")


    val credentialsProvider: BasicCredentialsProvider = new BasicCredentialsProvider
    credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(username, password))
    val clientCallback: RestClientBuilder.HttpClientConfigCallback = new RestClientBuilder.HttpClientConfigCallback {
      override def customizeHttpClient(httpAsyncClientBuilder: HttpAsyncClientBuilder): HttpAsyncClientBuilder = httpAsyncClientBuilder.setDefaultCredentialsProvider(credentialsProvider)
    }

    val requestCallback: RestClientBuilder.RequestConfigCallback = new RestClientBuilder.RequestConfigCallback {
      override def customizeRequestConfig(builder: RequestConfig.Builder): RequestConfig.Builder = {
        builder.setConnectTimeout(Duration.parse(connectionTimeout).toMillis().toInt)
        builder.setSocketTimeout(Duration.parse(readTimeout).toMillis().toInt)
        builder
      }
    }
    (hosts, clientCallback, requestCallback)
  }


  private def getClientBuilder(clusterName: String): RestClientBuilder = {
    val tuple = getConfig(clusterName)

    val hosts = tuple._1
    val clientCallback = tuple._2
    val requestCallback = tuple._3

    val builder = RestClient.builder(hosts: _*)
    builder.setHttpClientConfigCallback(clientCallback)
    builder.setRequestConfigCallback(requestCallback)
    builder
  }

  def getRestHighLevelClient(clusterName: String): RestHighLevelClient = {
    val builder = getClientBuilder(clusterName)
    new RestHighLevelClient(builder)
  }

  def getRestClient(clusterName: String): RestClient = {
    getClientBuilder(clusterName).build()
  }

}
