package com.winhc.bigdata.flink.func


import com.winhc.bigdata.flink.config.HbaseConfig
import com.winhc.bigdata.flink.java.entity.{PutCollection, SealedPut}
import org.apache.flink.configuration.{ConfigOptions, Configuration}
import org.apache.flink.runtime.state.{FunctionInitializationContext, FunctionSnapshotContext}
import org.apache.flink.streaming.api.checkpoint.CheckpointedFunction
import org.apache.flink.streaming.api.functions.sink.{RichSinkFunction, SinkFunction}
import org.apache.hadoop.hbase.TableName
import org.apache.hadoop.hbase.client.{BufferedMutator, Connection, ConnectionFactory, RetriesExhaustedWithDetailsException}
import java.util.concurrent.atomic.AtomicReference

import org.apache.flink.api.java.utils.ParameterTool

import scala.collection.JavaConverters.asScalaSetConverter

/**
 * @author: XuJiakai
 * @date: 2021/9/1 17:52
 */
class HbaseSinkFunction() extends RichSinkFunction[PutCollection] with CheckpointedFunction with BufferedMutator.ExceptionListener {
  var connection: Connection = _
  private val failureThrowable: AtomicReference[Throwable] = new AtomicReference[Throwable]

  override def open(parameters: Configuration): Unit = {
    val config=getRuntimeContext.getExecutionConfig.getGlobalJobParameters.asInstanceOf[ParameterTool]

    val configuration = HbaseConfig.getHbaseConfiguration(config, Seq(
      "winhc.hbase.config.hbase.zookeeper.quorum",
      "winhc.hbase.config.hbase.client.scanner.timeout.period",
      "winhc.hbase.config.hbase.client.retries.number",
      "winhc.hbase.config.hbase.client.pause",
      "winhc.hbase.config.hbase.client.max.perserver.tasks",
      "winhc.hbase.config.hbase.client.max.perregion.tasks",
      "winhc.hbase.config.hbase.client.keyvalue.maxsize",
      "winhc.hbase.config.hbase.client.ipc.pool.size",
      "winhc.hbase.config.zookeeper.recovery.retry"
    ))
    connection = ConnectionFactory.createConnection(configuration)
  }

  //TODO 按批写入
  override def invoke(value: PutCollection, context: SinkFunction.Context): Unit = {
    value
      .getData
      .entrySet()
      .asScala.map(e => (e.getKey, e.getValue))
      .par
      .foreach(t => {
        val (k, v) = t
        val table = connection.getTable(TableName.valueOf(k))
        table.put(SealedPut.toPuts(v.values()))
        table.close()
      })
  }

  override def close(): Unit = {
    if (connection != null) {
      connection.close()
    }
  }

  private def checkErrorAndRethrow(): Unit = {
    val cause = failureThrowable.get
    if (cause != null) throw new RuntimeException("An error occurred in HbaseSink.", cause)
  }

  override def snapshotState(context: FunctionSnapshotContext): Unit = {
    checkErrorAndRethrow()
  }

  override def initializeState(context: FunctionInitializationContext): Unit = {}

  override def onException(e: RetriesExhaustedWithDetailsException, bufferedMutator: BufferedMutator): Unit = {
    failureThrowable.compareAndSet(null, e)
  }
}
