package com.winhc.bigdata.flink.jobs

import com.winhc.bigdata.flink.java.constant.EnvConst
import com.winhc.bigdata.flink.jobs.company_job_step._
import com.winhc.bigdata.flink.source.KafkaSourceBuilder
import com.winhc.bigdata.flink.streaming.trigger.TimeCountTrigger
import org.apache.flink.api.scala._
import org.apache.flink.streaming.api.scala.{DataStream, StreamExecutionEnvironment}
import org.apache.flink.streaming.api.windowing.assigners.SlidingProcessingTimeWindows
import org.apache.flink.streaming.api.windowing.time.Time
import org.apache.flink.streaming.api.windowing.windows.TimeWindow

/**
 * @author: XuJiakai
 * @date: 2021/9/1 11:34
 */
object CompanyStreamJob {

  def main(args: Array[String]): Unit = {
    val env = StreamExecutionEnvironment.getExecutionEnvironment

    // 全局Job参数配合Rich函数自动配置
    env.getConfig.setGlobalJobParameters(EnvConst.createParameterTool(args))

    val kafkaSource = KafkaSourceBuilder.buildSourceFunction("xjk_test_topic")
    val source: DataStream[String] = env.fromSource[String](kafkaSource, CustomWatermarkStrategy(), "Kafka Source")


    source.winhcTransform().keyBy(o=>o.companyId).window(SlidingProcessingTimeWindows.of(Time.seconds(15), Time.seconds(10)))
      .trigger(TimeCountTrigger.processingTime[TimeWindow](2000))

    System.out.println(env.getExecutionPlan)
    //    env.execute("CompanyStreamJob")
  }
}
