package com.winhc.bigdata.flink.jobs.company_job_step

import com.winhc.bigdata.flink.event.{RowBaseDimEntity, UpdateEntity, ValidationExceptionEntity}
import com.winhc.bigdata.flink.java.entity.CompanyDataReceive
import com.winhc.bigdata.flink.jobs.company_job_step.OutputTags._
import com.winhc.bigdata.flink.utils.{BaseUtils, RowDataUtils}
import org.apache.flink.api.scala._
import org.apache.flink.streaming.api.functions.ProcessFunction
import org.apache.flink.streaming.api.scala.DataStream
import org.apache.flink.util.Collector
import org.json4s.JsonAST.JObject
import org.json4s.jackson.JsonMethods._
import org.slf4j.LoggerFactory

import scala.collection.JavaConverters._
import scala.collection.mutable

/**
 * @author ZhangJi
 * @since 2021-10-18 17:21
 */
object gs_01_kafka_transform_and_validation {
  private val logger = LoggerFactory.getLogger(classOf[gs_01_kafka_transform_and_validation])
}

case class gs_01_kafka_transform_and_validation(source: DataStream[String]) {


  def transform_and_validation(): DataStream[UpdateEntity] = {
    source
      .process(new ProcessFunction[String, UpdateEntity]() {
        override def processElement(value: String, ctx: ProcessFunction[String, UpdateEntity]#Context, out: Collector[UpdateEntity]): Unit = {
          try {
            val c = BaseUtils.objectmapper.readValue(value, classOf[CompanyDataReceive])
            try {
              val meta = c.getMeta
              val data = c.getData
              val base = c.getCompany
              val joMap: Map[String, mutable.Buffer[JObject]] = data.asScala
                .map(e => (e._1, e._2.asScala.map(fromJsonNode(_).asInstanceOf[JObject])))
                .toMap

/*              val validationResult = RowDataUtils.validateDims(joMap)
              val error = validationResult.error.filter(t => t._2.nonEmpty)
              if (error.nonEmpty) ctx.output(ValidationErrorTag, error)*/
//              val dims=joMap
              val dims = joMap

                .filter(t => t._2.nonEmpty)
                .map(t => (t._1, RowBaseDimEntity.ofJObject(t, (meta.getCompanyId, meta.getCompanyName))))
              if (dims.nonEmpty) {

                val entity = RowDataUtils.validateCompany(base) match {
                  case Right(c) => UpdateEntity(fromJsonNode(c).asInstanceOf[JObject], dims)
                  case Left(c) =>
                    val companyDataReceive = new CompanyDataReceive()
                    companyDataReceive.setCompany(c)
                    val rte = new RuntimeException("基本信息不合法")
                    ctx.output(ExceptionTag, ValidationExceptionEntity.of(rte, companyDataReceive))
                    UpdateEntity(null, dims)
                }
                if (base == null) {
                  UpdateEntity(null, dims)
                } else {
                  UpdateEntity(fromJsonNode(base).asInstanceOf[JObject], dims)
                }
                out.collect(entity)
              }
            } catch {
              case e: Exception =>
                gs_01_kafka_transform_and_validation.logger.error("error", e)
                ctx.output(ExceptionTag, ValidationExceptionEntity.of(e, c))
            }
          } catch {
            case e: Exception =>
              ctx.output(TransformErrorTag, ValidationExceptionEntity.of(e, value))
          }
        }
      })
  }
}
