package com.winhc.bigdata.flink.sink

import com.alibaba.fastjson.JSONObject
import com.winhc.bigdata.flink.config.ElasticsearchConfig
import org.apache.flink.api.common.functions.RuntimeContext
import org.apache.flink.elasticsearch6.shaded.org.elasticsearch.action.index.IndexRequest
import org.apache.flink.elasticsearch6.shaded.org.elasticsearch.action.update.UpdateRequest
import org.apache.flink.elasticsearch6.shaded.org.elasticsearch.client.RestClientBuilder
import org.apache.flink.elasticsearch6.shaded.org.elasticsearch.common.xcontent.XContentType
import org.apache.flink.streaming.connectors.elasticsearch.{ElasticsearchSinkFunction, RequestIndexer}
import org.apache.flink.streaming.connectors.elasticsearch6.{ElasticsearchSink, RestClientFactory}

import scala.collection.JavaConversions

/**
 * @author: XuJiakai
 * @date: 2021/9/1 17:19
 */
object ElasticsearchSinkBuilder {
  case class JsonObjectElasticsearchSinkFunction[T](index: String, docType: String, handle: (T) => (String, JSONObject)) extends ElasticsearchSinkFunction[T] {
    override def process(t: T, runtimeContext: RuntimeContext, indexer: RequestIndexer): Unit = {
      val tuple = handle(t)
      val id = tuple._1
      val json: JSONObject = tuple._2
      if (id != null) {
        val updateRequest = new UpdateRequest(index, docType, id)
          .doc(json, XContentType.JSON)
          .upsert(json, XContentType.JSON)
        indexer.add(updateRequest)
      } else {
        val indexRequest: IndexRequest = new IndexRequest(index, docType, id)
          .source(json, XContentType.JSON)
        indexer.add(indexRequest)
      }
    }
  }

  def buildSinkFunction[T](clusterName: String, index: String, docType: String, handle: (T) => (String, JSONObject)): ElasticsearchSink[T] = {

    val tuple = ElasticsearchConfig.getConfig(clusterName)
    val hosts = tuple._1
    val clientCallback = tuple._2
    val requestCallback = tuple._3

    val esSinkBuilder = new ElasticsearchSink.Builder[T](JavaConversions.seqAsJavaList(hosts), JsonObjectElasticsearchSinkFunction(index, docType, handle))

    esSinkBuilder.setBulkFlushMaxActions(500)

    // provide a RestClientFactory for custom configuration on the internally created REST client
    esSinkBuilder.setRestClientFactory(new RestClientFactory {
      override def configureRestClientBuilder(restClientBuilder: RestClientBuilder): Unit = {
        restClientBuilder.setHttpClientConfigCallback(clientCallback)
        restClientBuilder.setRequestConfigCallback(requestCallback)
      }
    })

    esSinkBuilder.build()
  }
}
