package com.winhc.bigdata.flink.utils

import com.fasterxml.jackson.databind.JsonNode
import com.fasterxml.jackson.databind.node.ObjectNode
import com.github.fge.jsonschema.main.JsonSchemaFactory
import com.winhc.bigdata.flink.config.ArgsCompanyJob
import com.winhc.bigdata.flink.event.{CompanyInfo, MetaInfo, ValidationResult}
import com.winhc.bigdata.flink.utils.BaseUtils.md5
import org.json4s.JsonAST.{JArray, JObject, JString}
import org.json4s.JsonDSL.jobject2assoc
import org.json4s.jackson.JsonMethods._
import java.io.{File, InputStream}

import scala.collection.JavaConverters._

/**
 * @author ZhangJi
 * @since 2021-10-13 14:54
 */
object RowDataUtils {
  val schemas: Map[String, JsonNode] = createAllSchema()

  def createAllSchema(): Map[String, JsonNode] = {
//    val schema_dir: String = this.getClass.getClassLoader.getResource("/schema").getPath

    Seq(
      ("company",
        """
          |{
          |  "$schema": "http://json-schema.org/draft-04/schema#",
          |  "type": ["null","object"],
          |  "properties": {
          |    "company_id": {
          |      "type": ["string"]
          |    },
          |    "new_cid": {
          |      "type": ["null","string"]
          |    },
          |    "base": {
          |      "type": ["null","string"]
          |    },
          |    "name": {
          |      "type": ["null","string"]
          |    },
          |    "name_en": {
          |      "type": ["null","string"]
          |    },
          |    "name_alias": {
          |      "type": ["null","string"]
          |    },
          |    "history_names": {
          |      "type": ["null","array"],
          |      "items": [
          |        {
          |          "type": ["string"]
          |        }
          |      ]
          |    },
          |    "legal_entity_id": {
          |      "type": ["null","string"]
          |    },
          |    "legal_entity_name": {
          |      "type": ["null","string"]
          |    },
          |    "legal_entity_type": {
          |      "type": ["null","integer"]
          |    },
          |    "reg_number": {
          |      "type": ["null","string"]
          |    },
          |    "company_org_type": {
          |      "type": ["null","string"]
          |    },
          |    "reg_location": {
          |      "type": ["null","string"]
          |    },
          |    "estiblish_time": {
          |      "type": ["null","string"]
          |    },
          |    "from_time": {
          |      "type": ["null","string"]
          |    },
          |    "to_time": {
          |      "type": ["null","string"]
          |    },
          |    "business_scope": {
          |      "type": ["null","string"]
          |    },
          |    "reg_institute": {
          |      "type": ["null","string"]
          |    },
          |    "approved_time": {
          |      "type": ["null","string"]
          |    },
          |    "reg_status": {
          |      "type": ["null","string"]
          |    },
          |    "reg_capital": {
          |      "type": ["null","string"]
          |    },
          |    "org_approved_institute": {
          |      "type": ["null","string"]
          |    },
          |    "parent_company_id": {
          |      "type": ["null","string"]
          |    },
          |    "company_type": {
          |      "type": ["null","integer"]
          |    },
          |    "credit_code": {
          |      "type": ["null","string"]
          |    },
          |    "org_number": {
          |      "type": ["null","string"]
          |    },
          |    "score": {
          |      "type": ["null","number"]
          |    },
          |    "cate_first_code": {
          |      "type": ["null","string"]
          |    },
          |    "cate_second_code": {
          |      "type": ["null","string"]
          |    },
          |    "cate_third_code": {
          |      "type": ["null","string"]
          |    },
          |    "lat": {
          |      "type": ["null","string"]
          |    },
          |    "lng": {
          |      "type": ["null","string"]
          |    },
          |    "province_code": {
          |      "type": ["null","string"]
          |    },
          |    "city_code": {
          |      "type": ["null","string"]
          |    },
          |    "county_code": {
          |      "type": ["null","string"]
          |    },
          |    "reg_capital_amount": {
          |      "type": ["null","integer"]
          |    },
          |    "reg_capital_currency": {
          |      "type": ["null","string"]
          |    },
          |    "actual_capital_amount": {
          |      "type": ["null","integer"]
          |    },
          |    "actual_capital_currency": {
          |      "type": ["null","string"]
          |    },
          |    "reg_status_std": {
          |      "type": ["null","string"]
          |    },
          |    "social_security_staff_num": {
          |      "type": ["null","integer"]
          |    },
          |    "cancel_date": {
          |      "type": ["null","string"]
          |    },
          |    "cancel_reason": {
          |      "type": ["null","string"]
          |    },
          |    "revoke_date": {
          |      "type": ["null","string"]
          |    },
          |    "revoke_reason": {
          |      "type": ["null","string"]
          |    },
          |    "emails": {
          |      "type": ["null","string"]
          |    },
          |    "phones": {
          |      "type": ["null","string"]
          |    },
          |    "wechat_public_num": {
          |      "type": ["null","string"]
          |    },
          |    "logo": {
          |      "type": ["null","string"]
          |    },
          |    "crawled_time": {
          |      "type": ["null","string"]
          |    },
          |    "create_time": {
          |      "type": ["null","string"]
          |    },
          |    "update_time": {
          |      "type": ["null","string"]
          |    },
          |    "deleted": {
          |      "type": ["null","integer"]
          |    }
          |  },
          |  "required": [
          |    "company_id"
          |  ]
          |}
          |""".stripMargin),
      ("company_holder",
        """{
          |  "$schema": "http://json-schema.org/draft-04/schema#",
          |  "type": ["null","object"],
          |  "properties": {
          |    "capital_actual": {
          |      "type": ["null","array"],
          |      "items": [
          |        {
          |          "type": ["object"],
          |          "properties": {
          |            "amomon": {
          |              "type": ["string"]
          |            },
          |            "paymet": {
          |              "type": ["null","string"]
          |            },
          |            "time": {
          |              "type": ["null","string"]
          |            }
          |          },
          |          "required": [
          |            "amomon"
          |          ]
          |        }
          |      ]
          |    },
          |    "amount": {
          |      "type": ["null","number"]
          |    },
          |    "capital": {
          |      "type": ["null","array"],
          |      "items": [
          |        {
          |          "type": ["object"],
          |          "properties": {
          |            "amomon": {
          |              "type": ["string"]
          |            },
          |            "paymet": {
          |              "type": ["null","string"]
          |            },
          |            "time": {
          |              "type": ["null","string"]
          |            }
          |          },
          |          "required": [
          |            "amomon"
          |          ]
          |        }
          |      ]
          |    },
          |    "company_id": {
          |      "type": ["string"]
          |    },
          |    "create_time": {
          |      "type": ["null","string"]
          |    },
          |    "holder_type": {
          |      "type": ["integer"]
          |    },
          |    "percent": {
          |      "type": ["null","number"]
          |    },
          |    "update_time": {
          |      "type": ["null","string"]
          |    },
          |    "deleted": {
          |      "type": ["null","integer"]
          |    },
          |    "company_name": {
          |      "type": ["null","string"]
          |    },
          |    "holder_id": {
          |      "type": ["string"]
          |    },
          |    "holder_name": {
          |      "type": ["string"]
          |    }
          |  },
          |  "required": [
          |    "company_id",
          |    "company_name",
          |    "holder_type",
          |    "holder_name"
          |  ]
          |}""".stripMargin),
      ("company_staff",
        """{
          |  "$schema": "http://json-schema.org/draft-04/schema#",
          |  "type": ["null","object"],
          |  "properties": {
          |    "staff_name": {
          |      "type": ["string"]
          |    },
          |    "hid": {
          |      "type": ["null","string"]
          |    },
          |    "update_time": {
          |      "type": ["null","string"]
          |    },
          |    "deleted": {
          |      "type": ["null","integer"]
          |    },
          |    "company_id": {
          |      "type": ["string"]
          |    },
          |    "staff_type": {
          |      "type": ["null","string"]
          |    },
          |    "create_time": {
          |      "type": ["null","string"]
          |    },
          |    "company_name": {
          |      "type": ["null","string"]
          |    }
          |  },
          |  "required": [
          |    "staff_name",
          |    "company_id"
          |  ]
          |}""".stripMargin)

    )
      //    new File(schema_dir).listFiles().filter(!_.isDirectory)
      //      .filter(t => t.toString.endsWith(".schema"))
      //      .map(f =>
      //        (f.getName.replace(".schema", ""), scala.io.Source.fromFile(f))
      //      )
      //      .map(t => {
      //        val (n, s) = t
      //        (n, try s.getLines().mkString("\n") finally s.close())
      //      })
      .map(t => {
        val (n, s) = t
        (n, asJsonNode(parse(s)))
      })
      .toMap
  }


  /**
   * TODO 未实现
   *
   * @param dim 维度名
   * @return rowkey
   */
  def generateRowKey(dim: String, json: JsonNode): String = {

    ArgsCompanyJob.job_args.get(dim).map(acj => {
      val md5_fields = acj.md5_fields
      val companyId = json.get("company_id").asText()
      if (md5_fields != null) {
        val fields = md5(md5_fields.map(field => json.get(field).asText()).filter(s => s != null && s.nonEmpty).mkString)
        s"${companyId}_$fields".stripMargin
      } else if (acj.rowkey_udf != null) {
        acj.rowkey_udf(companyId, json)
      } else {
        null
      }
    }).orNull
  }

  /**
   * 生成元数据
   *
   * @param dim         维度名
   * @param companyId   公司id
   * @param companyName 公司名
   * @return
   */
  def generateMetaInfo(dim: String, companyId: String, companyName: String): MetaInfo = {
    val m = ArgsCompanyJob.job_args.get(dim).orNull
    if (m == null) {
      MetaInfo(Some(CompanyInfo(companyId, companyName)), None, None, None)
    } else {
      MetaInfo(Some(CompanyInfo(companyId, companyName)), Some(m.esInfo), Some(m.hbaseInfo), Some(m.holoInfo))
    }
  }


  /**
   * 实体类校验
   *
   * @param dims 维度数据
   */
  def validateDims(dims: Map[String, Seq[JObject]]): ValidationResult = {
    val group = dims.flatMap(t => {
      val (k, v) = t
      if (!schemas.contains(k)) {
        Seq(Left((k, v.map(j => {
          j ~ ("error" -> JString("维度不存在"))
        }))))
      } else {
        val validator = JsonSchemaFactory.byDefault().getValidator
        val data = v.map(j => asJsonNode(j))
          .map(j => (validator.validate(schemas(k), j), j))
          .map(t => {
            val (p, j) = t
            (p, fromJsonNode(j).asInstanceOf[JObject])
          })
          .groupBy(t => {
            val (p, _) = t
            p.isSuccess
          })
        val successList = data.get(true).map(s => s.map(_._2)).getOrElse(Seq.empty)
        val errorList = data.get(false).map(s =>
          s.map(t => {
            val (p, j) = t
            val errors = p.iterator().asScala.map(m => fromJsonNode(m.asJson())).toList
            j ~ ("error" -> JArray(errors))
          })
        )
          .getOrElse(Seq.empty)
        Seq(Right((k, successList)), Left((k, errorList)))
      }
    }
    )
      .groupBy {
        case Right(_) => true
        case Left(_) => false
      }
    val success = group.getOrElse(true, Seq.empty).map {
      case Right(x) => x
      case _ => null
    }
      .filter(_ != null)
      .toMap
    val error = group.getOrElse(false, Seq.empty).map {
      case Left(x) => x
      case _ => null
    }
      .filter(_ != null)
      .toMap
    ValidationResult(success = success, error = error)
  }

  def validateCompany(company: JsonNode): Either[JsonNode, JsonNode] = {
    if (company == null) return Right(null)
    val companySchema = schemas("company")
    val validator = JsonSchemaFactory.byDefault().getValidator
    val p = validator.validate(companySchema, company)
    if (p.isSuccess) {
      Right(company)
    } else {
      val error = p.iterator().asScala.map(m => m.asJson()).toSeq.asJava
      company.asInstanceOf[ObjectNode].putArray("error").addAll(error)
      Left(company)
    }
  }

  def main(args: Array[String]): Unit = {
println("")
  }

}
